
#include once ".\libraries\sgui\sgui.bas"
#include once ".\libraries\sgui\clickmapgadget.bas"
#include once ".\libraries\sgui\menu.bas"
#include once ".\libraries\sgui\messagebox.bas"
#include once ".\libraries\stringlist.bi"
#include once ".\libraries\colors.bi"

#include once ".\book.bas"
#include once ".\chess.bas"
#include once ".\pieces.bas"
#include once ".\version.bas"

using sGUI
using strlist

extern "C" lib ".\engine"
  declare function EngineMove stdcall alias "EngineMove" (byval positionData as string) as string
end extern

declare sub DisplayArbitratorMessage(byval as string = "")
declare function IsPromotionMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsCastlingMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsEnPassantMove(byref as chess.tChessGame, byval as string) as boolean
declare function ReplaceFieldName(byval as string) as string
declare function ComparePiecePlacement(byval as string, byval as string) as boolean

screenres 320, 18 + 320 + 18, 32,, 4
windowtitle ""
const textHeight = 14
dim as integer w, h
screeninfo w, h
width w \ 8, h \ textHeight

dim shared as uinteger messageTextColor
dim shared as uinteger messageColor

customColors = 1
backgroundColor	= white
textColor	= black
gadgetColor	= yellow
gadgetGlowColor	= magenta
gadgetTextColor	= black
cursorColor = magenta
menuColor = lightyellow
menuTextColor = black
menuHiliteColor = gadgetColor
menuGhostedColor = ShiftRGB(MenuColor, -contrast / 2)
messageTextColor = textColor
messageColor = menuColor

graph.boardLeft = 0
graph.boardTop = 18

InitGFX
DisplayArbitratorMessage()

dim as string _
  sNew, sFlip, sQuit, sCheck, sWhiteToMove, sBlackToMove, sAutoplay, sMove, _
  sState(gsCheckmate to gsDraw), sMoves, sBoard, sHelp, sAbout, _
  sPromotion, sKnight, sBishop, sRook, sQueen, sHelpText, sAboutText

#include "language.bas"

dim as EventHandle ptr event = new EventHandle
dim as gadget ptr boardClickSensor
boardClickSensor = AddClickMapGadget(event, graph.boardLeft, graph.boardTop, ".\images\ClickMap.bmp")
GadgetOn(boardClickSensor)

dim as integer field_ID

type chessfield 
  fieldname as string 
  selected as integer 
  posx as integer 
  posy as integer 
end type

dim as chessfield Board(1 to 64)

for k as integer = 0 to 7 
  Board(1 + k * 8).fieldname = "a" & str(k + 1) 
  Board(2 + k * 8).fieldname = "b" & str(k + 1) 
  Board(3 + k * 8).fieldname = "c" & str(k + 1) 
  Board(4 + k * 8).fieldname = "d" & str(k + 1) 
  Board(5 + k * 8).fieldname = "e" & str(k + 1) 
  Board(6 + k * 8).fieldname = "f" & str(k + 1) 
  Board(7 + k * 8).fieldname = "g" & str(k + 1) 
  Board(8 + k * 8).fieldname = "h" & str(k + 1) 
next k

dim as boolean autoPlay = true
dim as integer autoColor = chess.cBlack

dim as integer _
  item_help, item_quit, item_about, item_move, item_auto, item_new, item_flip, _
  item_knight, item_bishop, item_rook, item_queen

Menu(event, 1, 0, 1, "Eschecs")
item_help = Menu(event, 1, 1, 1, sHelp)
item_quit = Menu(event, 1, 2, 1, sQuit)
item_about = Menu(event, 1, 3, 1, sAbout)
Menu(event, 2, 0, 1, sMoves)
item_move = Menu(event, 2, 1, 1, sMove)
item_auto = Menu(event, 2, 2, iif(autoplay, 3, 1), sAutoplay)
Menu(event, 3, 0, 1, sBoard)
item_new = Menu(event, 3, 2, 1, sNew)
item_flip = Menu(event, 3, 1, 1, sFlip)
Menu(event, 4, 0, 1, sPromotion)
item_knight = Menu(event, 4, 1, 1, sKnight)
item_bishop = Menu(event, 4, 2, 1, sBishop)
item_rook = Menu(event, 4, 3, 1, sRook)
item_queen = Menu(event, 4, 4, 3, sQueen)
MenuOn(event)

graph.upsideDown = false

dim startingPosition as string = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"

dim set as graph.T3

set.Create()
set.ChangePieces(mid(startingPosition, 1, instr(startingPosition, " ") - 1))
set.RedrawPieces()

dim game as chess.tChessGame
dim move as string = ""
dim state as tGameState = gsCheckmate
dim check as boolean = false
dim history as stringlist
dim journal as stringlist
dim rookMove as string = ""
dim bookLoaded as boolean = (LoadBook() = 0)
dim promotion as integer
dim promotionCommand as string = "wq"
dim oneMove as boolean = false
dim moveLabel as string

#macro UpdateAll()
history.Add(game.FEN)
game.GenerateGeometricalMoves()
game.GenerateSpecialMoves()
game.CheckMovesLegality()
state = game.GameState
if state = gsInProgress then
  var samePlacement = 0
  for i as integer = 0 to history.Count - 2
    if ComparePiecePlacement(history.Strings(i), history.Strings(history.Count - 1)) then
      samePlacement += 1
    end if
  next
  if samePlacement = 2 then
    state = gsDraw
  end if
end if
if len(move) = 4 then
  set.MovePiece(move)
end if
if rookMove <> "" then
  set.MovePiece(rookMove)
end if
if state = gsInProgress then
  check = game.Check
  DisplayArbitratorMessage(iif(check, sCheck & " ", "") & iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
else
  DisplayArbitratorMessage(sState(state))
end if
#endmacro

#macro ManageLegalMove()
if IsPromotionMove(game, move) then
  if moveLabel = "user" then
    if GetChecked(event, 4, 1) then
      promotion = chess.cKnight
    elseif GetChecked(event, 4, 2) then
      promotion = chess.cBishop
    elseif GetChecked(event, 4, 3) then
      promotion = chess.cRook
    elseif GetChecked(event, 4, 4) then
      promotion = chess.cQueen
    end if
    promotionCommand[0] = iif(game.ActiveColor = chess.cWhite, asc("w"), asc("b"))
    select case as const promotion
      case chess.cKnight
        promotionCommand[1] = asc("n")
      case chess.cBishop
        promotionCommand[1] = asc("b")
      case chess.cRook
        promotionCommand[1] = asc("r")
      case chess.cQueen
        promotionCommand[1] = asc("q")
    end select
  else
    promotion = chess.cQueen
    promotionCommand[0] = iif(game.ActiveColor = chess.cWhite, asc("w"), asc("b"))
    promotionCommand[1] = asc("q")
  end if
  game.DoMove(move, promotion)
  set.Promote(left(move, 2), promotionCommand)
else
  if IsCastlingMove(game, move) then
    select case move
      case "e1g1"
        rookMove = "h1f1"
      case "e1c1"
        rookMove = "a1d1"
      case "e8g8"
        rookMove = "h8f8"
      case "e8c8"
        rookMove = "a8d8"
    end select
  elseif IsEnPassantMove(game, move) then
    set.HidePiece(chr(move[2], move[1]))
  end if
  game.DoMove(move)
end if
journal.Add(move & " " & moveLabel)
#endmacro

game.Initialize(startingPosition)
UpdateAll()

do
  event->xSleep(1)
  
  if cbool(state = gsInProgress) andalso (oneMove or (autoPlay andalso cbool(autoColor = game.activeColor))) then
    oneMove = false
    var opponentMoveFound = false
    if BookLoaded then
      move = BookMove(game.EPD)
      if cbool(len(move) = 4) andalso game.IsLegal(move) then
        opponentMoveFound = true
        moveLabel = "book"
        ManageLegalMove()
        UpdateAll()
      end if
      move = ""
      rookMove = ""
    end if
    if not opponentMoveFound then
      move = EngineMove(game.FEN)
      if cbool(len(move) = 4) andalso game.IsLegal(move) then
        opponentMoveFound = true
        moveLabel = "engine"
        ManageLegalMove()
        UpdateAll()
      end if
      move = ""
      rookMove = ""
    end if
    if not opponentMoveFound then
      autoPlay = false
      if GetChecked(event, 2, 2) then Menu(event, 2, 2, 1, sAutoplay)
      DisplayArbitratorMessage(iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
    end if
  end if
  
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case boardClickSensor
        if state = gsInProgress then
          field_ID = getClickMapColor(boardClickSensor)
          move += iif(graph.upsideDown, ReplaceFieldName(Board(field_ID).fieldname), Board(field_ID).fieldname)
          if len(move) = 2 then
            if not game.IsLegalStartingSquare(move) then
              beep
              move = ""
            end if
          end if
          if len(move) = 4 then
            if game.IsLegal(move) then
              moveLabel = "user"
              ManageLegalMove()
              UpdateAll()
            end if
            move = ""
            rookMove = ""
          end if
        end if
    end select
  end if
  
  if event->MENUMESSAGE then
    select case event->MENUMESSAGE
      case item_help
        MessageBox(10, 18 + 10, sHelpText, sHelp, MBType_OK)
      case item_quit
        event->EXITEVENT = 1
      case item_about
        MessageBox(10, 18 + 10, sAboutText, sAbout, MBType_OK)
      case item_move
        if autoPlay then
          autoColor = chess.cBlack * autoColor
        else
          oneMove = true
        end if
      case item_auto
        if GetChecked(event, 2, 2) then
          Menu(event, 2, 2, 1, sAutoplay)
        else
          Menu(event, 2, 2, 3, sAutoplay)
        end if
        autoPlay = GetChecked(event, 2, 2)
      case item_new
        set.ClearBoard()
        set.ChangePieces()
        set.RedrawPieces()
        game.Initialize()
        history.ClearList
        UpdateAll()
      case item_flip
        set.ClearBoard()
        graph.upsideDown = not graph.upsideDown
        for i as integer = 0 to set.number - 1
          set.array(i).img.fSX = graph.ScrX(set.array(i).fX)
          set.array(i).img.fSY = graph.ScrY(set.array(i).fY)
        next i
        set.RedrawPieces()
      case item_knight
        Menu(event, 4, 1, 3, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_bishop
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 3, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_rook
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 3, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_queen
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 3, sQueen)
    end select
  end if
loop until event->EXITEVENT

delete event
if bookLoaded then CloseBook()
set.Free
journal.SaveToFile("eschecs.log")
end

sub DisplayArbitratorMessage(byval s as string = "")
  line (0, 18 + 320)-(320, 18 + 320 + 18), messageColor, BF
  draw string (8, 18 + 320 + (18 - textHeight) \ 2), s, messageTextColor
end sub

function IsPromotionMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

function IsCastlingMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cKing) andalso (Abs(m[0] - m[2]) = 2)
end function

function IsEnPassantMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((abs(g.board(m[2] - 96, m[3] - 48)) = chess.cNothing) and (m[0] <> m[2]))
end function

function ReplaceFieldName(byval s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  return chr(asc1 + 96, asc2 + 48)
end function

function ComparePiecePlacement(byval FENRecord1 as string, byval FENRecord2 as string) as boolean
  return mid(FENRecord1, 1, instr(FENRecord1, " ")) = mid(FENRecord2, 1, instr(FENRecord2, " "))
end function
