
#define RES_PATH "images\"
#define SQUARE_WIDTH 40
#define SQUARE_HEIGHT 40
#define MOVE_STEP 5
#define DELAY 10
#define STD_PLACEMENT "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR"
#define BOARD_WIDTH 320
#define BOARD_HEIGHT 320
#define MAX_PIECES 32

namespace graph

dim shared as integer boardLeft = 40
dim shared as integer boardTop = 40
dim shared as boolean upsideDown = false

function ScrX(x as integer) as integer
  if upsideDown then
    function = SQUARE_WIDTH * (8 - x) + boardLeft
  else
    function = SQUARE_WIDTH * (x - 1) + boardLeft
  end if
end function

function ScrY(y as integer) as integer
  if upsideDown then
    function = SQUARE_HEIGHT * (y - 1) + boardTop
  else
    function = SQUARE_HEIGHT * (8 - y) + boardTop
  end if
end function

type T1
  as any ptr fBack, fFore
  as integer fSX, fSY
  declare sub Create()
  declare sub Reload(byval aFilename as string)
  declare sub Free()
  declare sub Redraw(byval aSX as integer, byval aSY as integer)
  declare sub Hide()
end type

sub T1.Create()
  fBack = ImageCreate(SQUARE_WIDTH, SQUARE_HEIGHT)
  fFore = ImageCreate(SQUARE_WIDTH, SQUARE_HEIGHT)
  fSX = -1
  fSY = -1
end sub

sub T1.Reload(byval aFilename as string)
  BLoad(RES_PATH & aFilename, fFore)
end sub

sub T1.Free()
  ImageDestroy(fBack)
  ImageDestroy(fFore)
end sub

sub T1.Redraw(byval aSX as integer, byval aSY as integer)
  ScreenLock
  if fSX <> -1 then
    Put (fSX, fSY), fBack, PSET
  end if
  fSX = aSX
  fSY = aSY
  Get (fSX, fSY)-(fSX + SQUARE_WIDTH - 1, fSY + SQUARE_HEIGHT - 1), fBack
  Put (fSX, fSY), fFore, TRANS
  ScreenUnlock
end sub

sub T1.Hide()
  Put (fSX, fSY), fBack, PSET
end sub

type T2
  as T1 img
  as integer fX, fY
  declare sub Create()
  declare sub Reload(byval aWhat as string)
  declare sub Free()
  declare sub Show overload(byval aX as integer, byval aY as integer)
  declare sub Show overload(byval aCoord as string)
  declare function MoveTo(byval aX as integer, byval aY as integer) as integer
end type

sub T2.Create()
  img.Create()
  fX = -1
  fY = -1
end sub

sub T2.Reload(byval aWhat as string)
  img.Reload(aWhat & ".bmp")
end sub

sub T2.Free()
  img.Free()
end sub

sub T2.Show overload(byval aX as integer, byval aY as integer)
  fX = aX
  fY = aY
  img.Redraw(ScrX(fX), ScrY(fY))
end sub

sub T2.Show overload(byval aCoord as string)
  dim as integer x, y
  x = aCoord[0] - Asc("a") + 1
  y = aCoord[1] - Asc("1") + 1
  Show(x, y)
end sub

function T2.MoveTo(byval aX as integer, byval aY as integer) as integer
  dim as integer dx, dy, sx, sy, res = 1
  dx = (aX - fX)
  dy = (aY - fY)
  if (dx * dy = 0) _
  or (Abs(dx) = Abs(dy)) _
  or (Abs(dx * dy) = 2) then
    res = 0
    if dx <> 0 then dx \= Abs(aX - fX)
    if dy <> 0 then dy \= Abs(aY - fY)
    if Abs(aX - fX) = 2 * Abs(aY - fY) then dx *= 2
    if Abs(aY - fY) = 2 * Abs(aX - fX) then dy *= 2
    if upsideDown then
      dx *= -1
      dy *= -1
    end if
    sx = ScrX(fX)
    sy = ScrY(fY)
    fX = aX
    fY = aY
    do
      sx += dx * MOVE_STEP
      sy -= dy * MOVE_STEP
      sleep DELAY
      img.Redraw(sx, sy)
    loop until (sx = ScrX(fX)) and (sy = ScrY(fY))
  end if
  return res
end function

type T3
  dim array(MAX_PIECES - 1) as T2
  dim what (MAX_PIECES - 1) as string*2
  dim where(MAX_PIECES - 1) as string*2
  dim number as integer = MAX_PIECES
  declare sub Create()
  declare sub Free()
  declare sub ChangePieces(byval aPlacement as string = STD_PLACEMENT)
  declare sub RedrawPieces()
  declare sub ClearBoard()
  declare function MovePiece overload(_
    byval aX1 as integer, _
    byval aY1 as integer, _
    byval aX2 as integer, _
    byval aY2 as integer _
 ) as integer
  declare function MovePiece overload(byval move as string) as integer
  declare function HidePiece overload(_
    byval aX1 as integer, _
    byval aY1 as integer _
 ) as integer
  declare function HidePiece overload(byval square as string) as integer
  declare sub Promote(byval aWhere as string, byval aWhat as string)
end type

sub T3.Create()
  dim board as any ptr
  board = ImageCreate(BOARD_WIDTH, BOARD_HEIGHT)
  BLoad RES_PATH & "board.bmp", board
  Put (boardLeft, boardTop), board, TRANS
  ImageDestroy board
  for i as integer = 0 to number - 1
    array(i).Create()
  next i
end sub

sub T3.Free()
  for i as integer = 0 to number - 1
    array(i).Free()
  next i
end sub

sub T3.ChangePieces(byval aPlacement as string = STD_PLACEMENT)
  dim as byte _
  bs = Asc("/"), _
  b1 = Asc("1"), _
  b8 = Asc("8"), _
  ba = Asc("a")
  dim as integer x = 1, y = 8, j = 0
  dim n as byte
  for i as integer = 0 to Len(aPlacement) - 1
    if aPlacement[i] = bs then
      x = 1
      y -= 1
    else
      if (aPlacement[i] >= b1) and (aPlacement[i] <= b8) then
        n = aPlacement[i] - b1 + 1
        do while n >= 1
          x += 1
          n -= 1
        loop
      else
        what(j) = Iif(aPlacement[i] < ba, "w", "b") & LCase(Chr(aPlacement[i]))
        where(j) = Chr(x + ba - 1, y + b1 - 1)
        array(j).Reload(what(j))
        x += 1
        j += 1
      end if
    end if
  next i
  number = j
end sub

sub T3.RedrawPieces()
  for i as integer = 0 to number - 1
    array(i).Show(where(i))
  next i
end sub

sub T3.ClearBoard()
  for i as integer = 0 to number - 1
    array(i).img.Hide()
  next i
end sub

function T3.MovePiece overload(_
  byval aX1 as integer, _
  byval aY1 as integer, _
  byval aX2 as integer, _
  byval aY2 as integer _
) as integer
  dim as integer i, found = 0, res = 1
  i = 0
  while (i <= MAX_PIECES - 1) and (found = 0)
    if (array(i).fX = aX2) and (array(i).fY = aY2) then
      found = 1
      array(i).img.Hide()
      
      array(i).fX = -1
      array(i).fY = -1
      
    else
      i += 1
    end if
  wend
  i = 0
  while (i <= MAX_PIECES - 1) and (res = 1)
    if (array(i).fX = aX1) and (array(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    res = array(i).MoveTo(aX2, aY2)
    if res = 0 then
      where(i) = Chr(aX2 + Asc("a") - 1, aY2 + Asc("1") - 1)
    end if
  end if
  return res
end function

function T3.MovePiece overload(byval move as string) as integer
  move = LCase(move)
  return MovePiece(_
    move[0] - Asc("a") + 1, _
    move[1] - Asc("1") + 1, _
    move[2] - Asc("a") + 1, _
    move[3] - Asc("1") + 1 _
 )
end function

function T3.HidePiece overload(_
  byval aX1 as integer, _
  byval aY1 as integer _
) as integer
  dim as integer i, bx, by, res = 1
  i = 0
  while (i <= MAX_PIECES - 1) and (res = 1)
    if (array(i).fX = aX1) and (array(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    array(i).img.Hide()
    
    array(i).fX = -1
    array(i).fY = -1
    
  end if
  return res
end function

function T3.HidePiece overload(byval square as string) as integer
  square = LCase(square)
  return HidePiece(_
    square[0] - Asc("a") + 1, _
    square[1] - Asc("1") + 1 _
 )
end function

sub T3.Promote(byval aWhere as string, byval aWhat as string)
  dim i as integer = 0
  do while (i <= number - 1) andalso (where(i) <> aWhere)
    i += 1  
  loop
  if i <= number - 1 then
    array(i).img.Hide()
    what(i) = aWhat
    array(i).img.Reload(what(i) & ".bmp")
    array(i).Show(where(i))
  end if
end sub

end namespace
